/*
 * 套件管理类
 * @constructor SFA
 * */
(function(win){
	//判断组件库是否第一次载入,如果是初始化组件库
	win.ASG = win.ASG || {};
	
	//SFA类构造函数
	function SFA(cfgs){
		var configs = cfgs || {};
		var downloadDir = "_downloads/";//压缩包存放的目录
		var docDir = "_doc/";//解压的目标目录
		this.getDownloadDir = function(){
			return downloadDir;
		}
		this.getDocDir = function(){
			return docDir;
		}
		//初始化
		this.init();
	}
	
	//对象方法定义
	SFA.prototype = {
		init : function(){
			this.tasks = {};//任务队列
		},
		download :function(appName,url,downLoadSuccess,loading){
			var that = this;
			var downloadDir = this.getDownloadDir();	//压缩包存放的目录
			var docDir = this.getDocDir();				//解压的目标根目录
			var decompressDir = docDir + appName + "_root";	//解压的具体位置
			var appDocDir = appName + "_doc"; //自动创建app的私有doc目录
			var filename = url.split("/");
			filename = filename[filename.length - 1];
			filename = downloadDir + filename;
			
			if(this.tasks[appName] === undefined){
				this.tasks[appName] = {};
			}
			
			this.tasks[appName]['appName'] = appName;
			this.tasks[appName]['url'] = url;
			if(this.tasks[appName]['downloader'] === undefined){
				//下载前检查有没有同名文件
				plus.io.resolveLocalFileSystemURL(filename,function(entry){
					if(entry.isFile){
						entry.remove(function(entry){
							console.log("有同名文件删除文件" + filename + "成功");
						},function(error){
							console.log("有同名文件删除文件" + filename + "失败");
						});
					}
				},function(error){
					console.log("没有发现同名压缩文件");
				});
				
				//解压前检查有没有同名目录
				plus.io.resolveLocalFileSystemURL(decompressDir,function(entry){
					if(entry.isDirectory){
						entry.removeRecursively(function(entry){
							console.log("删除目录" + decompressDir + "成功,开始解压");
						},function(error){
							console.log("删除目录" + decompressDir + "失败");
						});
					}
				},function(error){
					console.log("没有发现同名目录,开始解压");
				});
				
				this.tasks[appName]['downloader'] = plus.downloader.createDownload(this.tasks[appName]['url'], {"filename" : downloadDir}, function ( d, status ) {
					// 下载完成
					if ( status == 200 ) {
						console.log("文件" + d.filename +"下载成功==================================================================");
						//下载的目录位置 d.filename 为downloadDir + 压缩包名字
						that.tasks[appName]['zipFilePath'] = d.filename;

						plus.zip.decompress(that.tasks[appName]['zipFilePath'], decompressDir,function() {
							console.log("解压到" + decompressDir + "成功");
							//执行成功的回调
							downLoadSuccess();
							
							//自动创建app私有doc目录
							plus.io.resolveLocalFileSystemURL(docDir,function(entry) {
								entry.getDirectory(appDocDir, {create:true,exclusive:false}, function( dir ){
									console.log("创建目录" + dir.name +"成功");
								}, function (e) {
									console.log("创建目录" + docDir +"失败");
								} );
							}, function (error) {
								console.log("打开docDir根目录失败");
							} );
							
							//准备删除压缩文件
							plus.io.resolveLocalFileSystemURL(that.tasks[appName]['zipFilePath'],function(entry){
								if(entry.isFile){
									entry.remove(function(entry){
										console.log("删除压缩文件成功");
									},function(error){
										console.log("删除压缩文件失败");
									});
								}
							},function(error){
								console.log("读取压缩文件失败");
							});

						},function(error) {
							console.log("解压到" + decompressDir + "失败");
						});
					} else {
						console.log("下载失败");
					}  
				});
				this.tasks[appName]['downloader'].addEventListener("statechanged",function(d,status){
					//console.log(d.state);
					that.tasks[appName]['state'] = d.state;
			    	if(d.state === undefined){
			    		//console.log("下载任务未开始");
			    	}
			    	if(d.state === 0){
			    		//console.log("下载任务开始调度");
			    	}
			    	if(d.state === 1){
			    		//console.log("下载任务开始请求");
			    	}
			    	if(d.state === 2){
			    		//console.log("下载任务请求已经接收");
			    	}
			    	if(d.state === 3){
			    		loading(d.downloadedSize,d.totalSize);
			    		//console.log("下载任务接收数据:" + d.downloadedSize + "/" + d.totalSize);
			    	}
			    	if(d.state === 4){
			    		that.tasks[appName]['downloader'] = undefined;//下载完成后销毁该对象
			    		//console.log("下载任务已完成:" + d.downloadedSize + "/" + d.totalSize);
			    	}
			    	if(d.state === 5){
			    		//console.log("下载任务已暂停");
			    	}
			    	if(d.state === -1){
			    		//console.log("枚举任务状态");
			    	}
			    });
				
				this.tasks[appName]['downloader'].start();
			}else{
				this.tasks[appName]['downloader'].start();
			}

		},
		pause : function(appName){
			this.tasks[appName]['downloader'].pause();
			console.log("暂停" + appName + "app的下载");
		},
		uninstall : function(appName){//卸载功能,会删除doc目录下面appName_root和appName_doc两个目录
			var that = this;
			var dirList = [],docDir;
			docDir = this.getDocDir();
			dirList.push(docDir + appName + "_root");
			dirList.push(docDir + appName + "_doc");
			
			//准备删除目录
			for(var i = 0; i<dirList.length; i++){
				(function(path){
					plus.io.resolveLocalFileSystemURL(path,function(entry){
						if(entry.isDirectory){
							entry.removeRecursively(function(entry){
								console.log("删除目录" + path + "成功");
							},function(error){
								console.log("删除目录" + path + "失败");
							});
						}
					},function(error){
						console.log("读取压缩文件失败");
					});
				})(dirList[i]);
			}
		},
		getAppDir : function(callback){
			var appName,that = this,len;
			this.appDir = [];//app名称列表
			plus.io.requestFileSystem( plus.io.PRIVATE_DOC, function(fs){
				// fs.root是根目录操作对象DirectoryEntry
				// 创建读取目录信息对象 
				var directoryReader = fs.root.createReader();
				directoryReader.readEntries( function( entries ){
					var i,path;
					if(entries.length === 0){
						callback && callback();
					}else{
						for( i=0; i < entries.length; i++ ) {
							path = plus.io.convertAbsoluteFileSystem(entries[i].fullPath);
							console.log("文件路径:" + path);
							(function(path,i){
								plus.io.resolveLocalFileSystemURL(path,function(entry){
									if(entry.isFile){//此处会删除_doc目录下面的文件,但不删除目录,_doc根目录下面不允许存在文件，只能存在目录
										entry.remove(function(entry){
											console.log("清理_doc目录下面的文件" + path + "成功");
										},function(error){
											console.log("清理_doc目录下面的文件" + path + "失败");
										});
									}
									if(entry.isDirectory){
										appName = entry.name.split("_");
										len = appName.length;
										if(len === 2 && appName[1] === "root"){
											that.appDir.push(appName[0]);
											//console.log("app名字" + appName[0]);
										}
									}
									if(i === entries.length-1){//目录获取完以后执行回调函数
										callback && callback();
									}
								},function(error){
									console.log("获取文件失败");
								});
							})(path,i)
						}
					}
				}, function ( e ) {
					console.log( "Read entries failed: " + e.message );
				} );
			} );
		},
		filterApp : function(callback){
			var appList = this.appDir,appListLen,that = this;
			var docDir = this.getDocDir();	//解压的目标根目录
			var versionFilePath,result;	//version.json文件位置
			this.appVersion = {};
			appListLen = appList.length;//此处获取appDir的长度,appDir的作用是获取目录下面**_root名称的目录,默认_前缀前部的名称为app名称
			
			if(appListLen === 0){
				callback && callback();
			}else{
				for(var j = 0; j < appListLen; j++){
					(function(j){
						versionFilePath = docDir + appList[j] + "_root/version.json";
						plus.io.resolveLocalFileSystemURL(versionFilePath,function(entry){
							entry.file(function(file){
								var reader = new plus.io.FileReader();
								reader.readAsText(file);
								reader.onloadend = function(e) {
									result = JSON.parse(e.target.result);
									that.appVersion[appList[j]] = {};
									that.appVersion[appList[j]]['version'] = result["version"];//app的版本号
									that.appVersion[appList[j]]['isNew'] = true;//是否是最新的版本
									that.appVersion[appList[j]]['isWarn'] = false;//是否需要提醒更新
									
									//console.log(result["version"]);
									if(j === appListLen - 1){
										callback && callback();
									}
								}
							});
							//console.log("获取json文件成功");
						},function(error){
							that.appVersion[appList[j]] = {};
							that.appVersion[appList[j]]['version'] = "";//app的版本号
							that.appVersion[appList[j]]['isNew'] = false;//是否是最新的版本
							that.appVersion[appList[j]]['isWarn'] = true;//是否需要提醒更新
							if(j === appListLen - 1){
								callback && callback();		
							}
							//console.log("获取json文件失败");
						})
					})(j)
				}
			}
		},
		checkState : function(){
			var tasks = this.tasks;
			var allTaskFinished = true;//默认任务都已经下载完毕
			for(var i in tasks){
				if(tasks[i]['state'] !== 4){
					allTaskFinished = false;
					break;
				}
			}
			return allTaskFinished;
		},
		clearDownloadDir : function(){
			
			plus.io.requestFileSystem( plus.io.PRIVATE_DOC, function(fs){
				// fs.root是根目录操作对象DirectoryEntry
				// 创建读取目录信息对象 
				var directoryReader = fs.root.createReader();
				directoryReader.readEntries( function( entries ){
					var i,path;
					console.log("删除前一共" + entries.length + "个");
					for( i=0; i < entries.length; i++ ) {
						path = plus.io.convertAbsoluteFileSystem(entries[i].fullPath);
						console.log("文件路径:" + path);
						(function(path){
							plus.io.resolveLocalFileSystemURL(path,function(entry){
								if(entry.isFile){
									entry.remove(function(entry){
										console.log("删除文件" + path + "成功");
									},function(error){
										console.log("删除文件" + path + "失败");
									});
								}
								if(entry.isDirectory){
									entry.removeRecursively(function(entry){
										console.log("删除目录" + path + "成功");
									},function(error){
										console.log("删除目录" + path + "失败");
									});
								}
	
							},function(error){
								console.log("获取文件失败");
							});
						})(path)
					}
				}, function ( e ) {
					console.log( "Read entries failed: " + e.message );
				} );
			} );
			
			//====测试
		},
		showProot : function(){
			plus.io.requestFileSystem( plus.io.PRIVATE_DOC, function(fs){
				// fs.root是根目录操作对象DirectoryEntry
				// 创建读取目录信息对象 
				var directoryReader = fs.root.createReader(),fileList = [];
				directoryReader.readEntries( function( entries ){
					var i,path;
					for( i=0; i < entries.length; i++ ) {
						fileList.push(entries[i].name);
					}
					console.log("项目根目录" + entries.length + "个文件:" + fileList.join());
				}, function ( e ) {
					console.log( "Read entries failed: " + e.message );
				} );
			} );
		}
	}
	
	//将Map类暴露在外部
	win.ASG.SFA = SFA;
})(window);